//
//  SupaLabel.h
//
//  Created by Konstantin Kiselyov on 9/23/12.
//  Copyright (c) 2012 Konstantin Kiselyov. All rights reserved.
//

#import <UIKit/UIKit.h>
#import <CoreText/CoreText.h>
#import <QuartzCore/QuartzCore.h>

typedef enum {
    SL_VERTICAL_ALIGNMENT_TOP,
    SL_VERTICAL_ALIGNMENT_CENTER,
    SL_VERTICAL_ALIGNMENT_BOTTOM
} SupaLabelVerticalAlignment_t;

typedef void (^FillSelectionCallback_t)(CGContextRef context, NSArray* selectionPaths, CGRect contextRect);

@interface SupaLabel : UILabel

@property (copy, nonatomic) NSAttributedString* attributedText;
@property (assign, nonatomic) SupaLabelVerticalAlignment_t verticalAlignment;
@property (assign, nonatomic) CTTextAlignment textAlignmentHor;
@property (assign, nonatomic) CTLineBreakMode lineBreakModeKind;
@property (assign, nonatomic) NSInteger columnsNumber;
@property (assign, nonatomic) CGFloat columnPadding;
@property (assign, nonatomic) CGFloat lineSpacing;
@property (strong, nonatomic) NSArray* clippingPaths;
@property (assign, nonatomic) CGFloat selectionCornerRadius;
@property (assign, nonatomic) UIEdgeInsets padding;
@property (strong, nonatomic) UIColor* selectionColor;
@property (copy, nonatomic) FillSelectionCallback_t fillSelectionCallback;


- (void)markTextWithPattern:(NSString *)pattern;
- (void)updateWithAttributedString:(NSAttributedString *)attrText_;

- (void)setFont:(UIFont *)font_ forRange:(NSRange)fontRange;
- (void)setForegroundColor:(UIColor *)color_ forRange:(NSRange)colorRange;

- (void)addClippingRect:(CGRect)clippingRect padding:(UIEdgeInsets)padding;
- (void)removeAllClippingPaths;

- (void)selectRange:(NSRange)selectionRange_;
- (void)deselectAll;

@end
